#ifndef __VCS_H__
#define __VCS_H__

#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include <SDL/begin_code.h>

#define NO_ROTATION	0
#define OFF			0
#define CLOCKWISE	1
#define CCLOCKWISE	2

/**
 *  This is the secret of VCS.  RotateImage.cpp uses the VCS_Surface
 *  struct.  The REAL member values are kept in OldSurface,
 *  and the base-class SDL_Surface members are filled with fake
 *  VCS-translated values.
 *
 *  When it's locked, however, the the surface is unrotated
 *  to match the fake values;  then re-rotated upon unlock.
 */
struct VCS_Surface:public SDL_Surface
{
  /* For use ONLY by VCS */
  int RotationMode;
  SDL_Surface *OldSurface;
  SDL_Surface *LockedSurface;
  int VCS_Locked;
};

#ifdef __cplusplus
extern "C" {
#endif

/* VCS specific functions */
DECLSPEC int SDLCALL          VCS_TranslateCoords(Sint16 *x, Sint16 *y);
DECLSPEC VCS_Surface *SDLCALL SDL_VCS(SDL_Surface *surface);
DECLSPEC SDL_Surface *SDLCALL VCS_RotateSurface(SDL_Surface *surface, bool autofree=true);
DECLSPEC SDL_Surface *SDLCALL VCS_UnRotateSurface(VCS_Surface *surface, bool autofree=true);
DECLSPEC Uint32	SDLCALL getpixel(SDL_Surface *surface, int x, int y);
DECLSPEC void	SDLCALL putpixel(SDL_Surface *surface,int x, int y, Uint32 pixel);

/*VCS conversions of SDL functions */
DECLSPEC VCS_Surface *SDLCALL VCS_ConvertSurface(VCS_Surface *ref,SDL_PixelFormat *format, Uint32 flags);
DECLSPEC VCS_Surface *SDLCALL VCS_CreateRGBSurface(Uint32 flags,Sint16 w,Sint16 h,Uint8 bpp,Uint32 rmask, Uint32 gmask, Uint32 bmask, Uint32 amask);
DECLSPEC VCS_Surface *SDLCALL VCS_SetVideoMode(Sint32 w, Sint32 h, Uint8 bpp,Uint32 flags);
DECLSPEC VCS_Surface *SDLCALL VCS_LoadBMP(const char *file);
DECLSPEC VCS_Surface *SDLCALL VCS_CloneSurface(VCS_Surface *surface, Uint32 w, Uint32 h);
DECLSPEC VCS_Surface *SDLCALL VCS_GetVideoSurface();
DECLSPEC VCS_Surface *SDLCALL VCS_DisplayFormat(VCS_Surface *surface);
DECLSPEC VCS_Surface *SDLCALL VCS_Load(const char *fname);

DECLSPEC void SDLCALL VCS_FreeSurface(VCS_Surface *surface);
DECLSPEC int  SDLCALL VCS_BlitSurface(VCS_Surface *source, SDL_Rect *srcrect, VCS_Surface *dest, SDL_Rect *dstrect);
DECLSPEC void SDLCALL VCS_FillRect(VCS_Surface *surface, SDL_Rect *rect, Uint32 color);
DECLSPEC void SDLCALL VCS_UpdateRect(VCS_Surface *screen,Sint32 x, Sint32 y, Sint32 w, Sint32 h);
DECLSPEC void SDLCALL VCS_UpdateRects(VCS_Surface *screen, int numrects, SDL_Rect *rects);

DECLSPEC void SDLCALL PrintSurface(SDL_Surface *s);
DECLSPEC void SDLCALL PrintRect(SDL_Rect *rect);
DECLSPEC void SDLCALL VCS_SetClipRect(VCS_Surface *surface, SDL_Rect *rect);
DECLSPEC void SDLCALL VCS_GetClipRect(VCS_Surface *surface, SDL_Rect *rect);
DECLSPEC void SDLCALL VCS_SetColorKey(VCS_Surface *surface, Uint32 flag, Uint32 color);
DECLSPEC void SDLCALL VCS_UnlockSurface(VCS_Surface *surface);
DECLSPEC void SDLCALL VCS_WM_SetIcon(VCS_Surface *icon, Uint8 *mask);
DECLSPEC void SDLCALL VCS_Quit(void);
DECLSPEC void SDLCALL VCS_SaveBMP(VCS_Surface *surface, const char *file);

DECLSPEC int SDLCALL VCS_PollEvent(SDL_Event *event);
DECLSPEC int SDLCALL VCS_Flip(VCS_Surface *surface);
DECLSPEC int SDLCALL FindColor(VCS_Surface *surface, SDL_Color *color);
DECLSPEC int SDLCALL VCS_PeepEvents(SDL_Event *events, int numevents, SDL_eventaction action, Uint32 mask);
DECLSPEC int SDLCALL VCS_SetColors(VCS_Surface *surface, SDL_Color *colors, int firstcolor, int ncolors);
DECLSPEC int SDLCALL VCS_LockSurface(VCS_Surface *surface);
DECLSPEC int SDLCALL VCS_SetAlpha(VCS_Surface *surface, Uint32 flag, Uint8 alpha);

DECLSPEC Uint8 SDLCALL VCS_GetMouseState(int *x, int *y);

#ifdef __cplusplus
}
#endif


///////////////////

#ifndef __VCS_CPP__

#undef  SDL_Surface
#define SDL_Surface           VCS_Surface

#undef  SDL_ConvertSurface
#define SDL_ConvertSurface    VCS_ConvertSurface
#undef  SDL_CreateRGBSurface
#define SDL_CreateRGBSurface  VCS_CreateRGBSurface
#undef  SDL_SetVideoMode
#define SDL_SetVideoMode      VCS_SetVideoMode
#undef  SDL_LoadBMP
#define SDL_LoadBMP           VCS_LoadBMP
#undef  SDL_GetVideoSurface
#define SDL_GetVideoSurface   VCS_GetVideoSurface
#undef  SDL_DisplayFormat
#define SDL_DisplayFormat     VCS_DisplayFormat
#undef  SDL_FreeSurface
#define SDL_FreeSurface       VCS_FreeSurface
#undef  SDL_BlitSurface
#define SDL_BlitSurface       VCS_BlitSurface
#undef  SDL_FillRect
#define SDL_FillRect          VCS_FillRect
#undef  SDL_UpdateRect
#define SDL_UpdateRect        VCS_UpdateRect
#undef  SDL_UpdateRects
#define SDL_UpdateRects       VCS_UpdateRects
#undef  SDL_SetClipRect
#define SDL_SetClipRect       VCS_SetClipRect
#undef  SDL_GetClipRect
#define SDL_GetClipRect       VCS_GetClipRect
#undef  SDL_SetColorKey
#define SDL_SetColorKey       VCS_SetColorKey
#undef  SDL_UnlockSurface
#define SDL_UnlockSurface     VCS_UnlockSurface
#undef  SDL_WM_SetIcon
#define SDL_WM_SetIcon        VCS_WM_SetIcon
#undef  SDL_Quit
#define SDL_Quit              VCS_Quit
#undef  SDL_SaveBMP
#define SDL_SaveBMP           VCS_SaveBMP
#undef  SDL_PollEvent
#define SDL_PollEvent         VCS_PollEvent
#undef  SDL_Flip
#define SDL_Flip              VCS_Flip
#undef  SDL_PeepEvents
#define SDL_PeepEvents        VCS_PeepEvents
#undef  SDL_SetColors
#define SDL_SetColors         VCS_SetColors
#undef  SDL_LockSurface
#define SDL_LockSurface       VCS_LockSurface
#undef  SDL_SetAlpha
#define SDL_SetAlpha          VCS_SetAlpha
#undef  SDL_GetMouseState
#define SDL_GetMouseState     VCS_GetMouseState
#undef  IMG_Load
#define IMG_Load              VCS_Load

//All VCS_Surface's must be locked to directly access, software or not
#undef SDL_MUSTLOCK
#define SDL_MUSTLOCK(surface) (1)

#endif/*__VCS_CPP__*/

#include <SDL/close_code.h>

#endif //__VCS_H
