/***************************************************************************
                          rect_ops.h  -  description
                             -------------------
    begin                : Wed Jun 11 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
                          rect_ops.h  -  description
                             -------------------
    begin                : Wed Jun 11 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Softw+are Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __RECT_OPS_H__
#define __RECT_OPS_H__

#ifndef __cplusplus
#error "Requires C++ compilation"
#endif

#include "rect.h"

inline bool AreaIsTransparent(SDL_Surface *surf, Rect r)
{
  int x,y;
  Uint32 transcolor=0;

  if(surf==NULL)  return(true);
  else if(!(surf->flags&SDL_SRCCOLORKEY))
  {
    return(false);
  }

  transcolor=surf->format->colorkey;
  
  SDL_LockSurface(surf);

  for(y=r.y; y<(r.y+r.h); y++)  
  for(x=r.x; x<(r.x+r.w); x++)
  {
    Uint32 c=getpixel(surf,x,y);
    if(c!=transcolor)
    {
      SDL_UnlockSurface(surf);
      return(false);
    }
  }

  SDL_UnlockSurface(surf);

  return(true);
}

inline Rect Rect::ShrinkBottom(int amount)
{
  if(amount<0)
    return(ExpandDown(-amount));
  else if(amount>h)
    Set(0,0,0,0);
  else
    h-=amount;

  return(*this);
}

inline Rect Rect::ShrinkTop(int amount)
{
  if(amount<0)  return(ExpandUp(-amount));
  if(amount>h)
    Set(0,0,0,0);
  else
  {
    y+=amount;
    h-=amount;
  }

  return(*this);
}

inline Rect Rect::ExpandDown(int amount, int max)
{
  if(amount<0)  return(ShrinkBottom(-amount));
  if(max>=0)
  {
    if(y+h+amount>max)
      h=(max-y);
    else
      h+=amount;
  }
  else
  {
    h+=amount;
  }

  return(*this);
}

inline Rect Rect::ExpandUp(int amount)
{
  if(amount<0)  return(ShrinkTop(-amount));
  else if(amount>=y)
  {
    h+=y;
    y=0;
  }
  else
  {
    h+=amount;
    y-=amount;
  }

  return(*this);
}


inline Rect Rect::Shrink(int border)
{
  if((w==0)||(h==0))
  {
    /* Do nothing */
  }
  if( ((border*2)>=w)|| ((border*2)>=h) )
  {
    Set(0,0,0,0); /* Zero out rect  */
  }
  else    /* Shrink the rectangle */
  {
    w-=(border*2);  /* Reduce width               */
    h-=(border*2);  /* Reduce height              */
    x+=border;      /* Offset X into new position */
    y+=border;      /* Offset Y into new position */
  }

  return(*this);
}

inline Rect Rect::ShrinkLeft(int amount)
{
  if(amount<0)
    return(ExpandLeft(-amount));
  else if(amount>w)
  {
    Set(0,0,0,0);
  }
  else
  {
    x+=amount;
    w-=amount;
  }

  return(*this);
}

inline Rect Rect::ExpandLeft(int amount)
{
  if(amount<0)  return(ShrinkLeft(-amount));
  
  x-=amount;
  w+=amount;

  if(x<0)
  {
     amount+=x;
     w+=amount;
     x=0;
  }

  return(*this);
}

inline Rect Rect::ShrinkRight(int amount)
{
  if(amount<0)
    return(ExpandRight(amount));
  else if(amount>=w)
    Set(0,0,0,0);
  else
    w-=amount;
    
  return(*this);
}

inline Rect Rect::ExpandRight(int amount, int max)
{
  if(amount<0)  return(ShrinkRight(-amount));
  else
  {
    if(max>=0)
    {
      if((x+w+amount)>max)
        w=max-x;
      else
        w+=amount;
    }
    else
    {
      w+=amount;
    }
  }
}

#endif/*__RECT_OPS_H__*/
