/***************************************************************************
                          rectlist.cpp  -  description
                             -------------------
    begin                : Fri Mar 7 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "rectlist.h"

/* True means OK, false means rect out of range */
bool Normalize(SDL_Rect &r,SDL_Surface *screen)
{
  if(r.x<0)
  {
    if(-r.x>r.w)
    {
      return(false);
    }

    r.w+=r.x;
    r.x=0;
  }

  if(r.y<0)
  {
    if(-r.y>r.h)
    {
      return(false);
    }

    r.h+=r.y;
    r.y=0;
  }

  if(r.x+r.w>screen->w)
  {
    if(r.x>=screen->w)
    {
      return(false);
    }
    r.w=screen->w-r.x;    
  }

  if(r.y+r.h>screen->h)
  {
    if(r.y>=screen->h)
    {
      return(false);
    }
    r.h=screen->h-r.y;
  }

  return(true);  
}

RectList::RectList(int numrects)
{
  size=numrects;
  rects = new Rect[numrects];
}

void RectList::Clear()
{
  pos=0;
}

bool RectList::Reduce(SDL_Surface *screen)
{
  bool reduced=false;
  int n=-1;
  
  for(n=0; n<pos; n++)
  {
    if(!Normalize(rects[n],screen))
    {
      Remove(n);
      reduced=true;
      n--;
    }
  }
  
  return(reduced);
}

void RectList::Update(SDL_Surface *screen)
{
  Reduce(screen);

  SDL_UpdateRects(screen,pos,rects);
}

bool RectList::Remove(int index)
{
  if(index>=pos)
  {
    return(false);
  }
  /* If it's the end one, just shrink stack */
  else if(index==(pos-1))    
  {
    pos--;
  }
  else
  { /* Else, move in one from the end */
    rects[index]=rects[pos-1];
    pos--;
  }
    
  return(true);
}

Rect RectList::operator[](int index)
{
  if(index>=pos)
  {
    return(Rect(0,0,0,0));
  }
  else
  {
    return(rects[index]);
  }
}

Rect RectList::Pop()
{
  if(pos==0)
  {
    return(Rect(0,0,0,0));
  }
  else
  {
    return(rects[--pos]);
  }
}

int RectList::Pos()
{
  return(pos);
}

int RectList::Size()
{
  return(size);
}

bool RectList::Push(Rect r)
{
  if(pos>=(size-1))
  {
    return(false);
  }
  else
  {    
    rects[pos++]=r;
    return(true);
  }
}

RectList::~RectList()
{
  delete []rects;
}
