/***************************************************************************
                          rectlist.h  -  description
                             -------------------
    begin                : Fri Mar 7 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __RECTLIST_H__
#define __RECTLIST_H__

#include "RotateImage.h"
#include "Fxp.h"
#include "rect.h"

#include <SDL/begin_code.h>

/*! \brief A simple stack class for rectangles              */
class DECLSPEC RectList
{
public:
  /*! \brief Constructor to specify size of stack           */
  RectList(int numrects=1024);
  /*! \brief Operator to make class act like an array       */
  Rect operator[](int);
  /*! \brief Pops a rectangle off the top of the stack      */
  Rect Pop();
  /*! \brief Updates the represented portions of the screen */  
  void Update(SDL_Surface *screen);
  /*! \brief Totally empties the stack                      */
  void Clear();
  /*! \brief Removes the selected rectangle from the stack  */
  bool Remove(int index);
  /*! \brief Pushes a rect onto the top of the stack        */
  bool Push(Rect r);
  /*! \brief Attempts to merge overlapping rectangles       */
  bool Reduce(SDL_Surface *screen);
  /*! \brief Returns the current position in the stack      */
  int Pos();
  /*! \brief Returns the number of elements in the stack    */
  int Size();
  /*! \brief Destructor, frees all data                     */  
  ~RectList();
  
private:
  /*! \brief Allocated array of rectangles                  */
  Rect *rects;
  /*! \brief Size of the array in elements                  */
  int   size;
  /*! \brief Current position in the array                  */
  int   pos;
};

#include <SDL/close_code.h>

#endif/*__RECTLIST_H__*/
