/***************************************************************************
                          spritelist.cpp  -  description
                             -------------------
    begin                : Fri Mar 7 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "spritelist.h"

SpriteList::SpriteList()
{
  head=NULL;
}

void SpriteList::Destroy()
{
  Sprite *ptr=head;
  while(ptr!=NULL)
  {
    Sprite *tmp=ptr->next;
    /* If it can be safely freed, do so */
    if(ptr->Deleteable())
    {
      delete ptr;
    }
    else  /* Otherwise just unlink from list */
    {
      ptr->prev=ptr->next=NULL;
    }

    ptr=tmp;
  }

  head=NULL;
}

SpriteList::~SpriteList()
{
  Destroy();
}

Sprite *SpriteList::Head()
{
  return(head);
}

Sprite *SpriteList::Tail()
{
  Sprite *spr=head;
  while(spr!=NULL)
  {
    if(spr->next==NULL) return(spr);
    spr=spr->next;
  }

  return(NULL);
}

bool SpriteList::Extract(Sprite *spr)
{
  if(spr==NULL)
  {
    return(false);
  }
  else if(!Find(spr))
  {
    return(false);
  }
  else if(spr==head)  
  {
    head=head->next;
    head->prev=NULL;
    spr->prev=spr->next=NULL;
    return(true);
  }
  else
  {
    if(spr->prev!=NULL) spr->prev->next=spr->next;
    if(spr->next!=NULL) spr->next->prev=spr->prev;
    spr->next=spr->prev=NULL;
    return(true);
  }
}

bool SpriteList::Insert(Sprite *spr)
{
  Sprite *ptr=NULL;
  if(spr==NULL)   return(false);
  if(head==NULL)  return(Prepend(spr));

  ptr=head;  
  while(ptr!=NULL)
  {
    if(ptr==spr)                                return(false);
    else if(spr->DrawLayer() <= ptr->DrawLayer())  break;
    
    ptr=ptr->next;
  }
  {
    Sprite *tmp=ptr;
    while(tmp!=NULL)
    {
      if(tmp==spr)  return(false);
      tmp=tmp->next;
    }
  }

  if(ptr==NULL)
  {
    return(Append(spr));
  }
  else if(ptr==head)
  {
    return(Prepend(spr));    
  }
  else
  {
    if(ptr->prev!=NULL) ptr->prev->next=spr;
    spr->prev=ptr->prev;
    spr->next=ptr;
    ptr->prev=spr;
    return(true);    
  }
}

bool SpriteList::Find(Sprite *spr)
{
  Sprite *ptr=NULL;
  if(spr==NULL) return(false);

  ptr=head;
  while(ptr!=NULL)
  {
    if(ptr==spr)
    {
      return(true);
    }
    /* TESTME - that > might actually be supposed to be a < */
    else if(ptr->DrawLayer() > spr->DrawLayer())
    {
      return(false);
    }
    ptr=ptr->next;
  }
  return(false);
}

bool SpriteList::Append(Sprite *spr)
{
  if(spr==NULL) return(false);
  if(head==NULL)
  {
    return(Prepend(spr));
  }
  else
  {
    Sprite *ptr=Tail();
    if(ptr==NULL) return(false);
    spr->prev=ptr;
    spr->next=NULL;
    ptr->next=spr;
    return(true);
  }
}

bool SpriteList::Prepend(Sprite *spr)
{    
  if(spr==NULL) return(false);
  if(head==NULL)
  {
    spr->prev=spr->next=NULL;
    head=spr;
    return(true);    
  }
  else
  {
    spr->prev=NULL;
    spr->next=head;
    head->prev=spr;
    head=spr;
    return(true);
  }
}
