/***************************************************************************
                          spritelist.h  -  description
                             -------------------
    begin                : Fri Mar 7 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SPRITELIST_H
#define SPRITELIST_H

#include "Fxp.h"
#include "rect.h"
#include "sprite.h"

#include <SDL/begin_code.h>

/**
  *@author Tyler Montbriand
  */

/*! \brief A doubly-linked sprite list class  */
class DECLSPEC SpriteList
{
public:
  /*! \brief Constructor  */
  SpriteList();
  /*! \brief Destructor, frees any freeable sprites and destroys list */
  ~SpriteList();
  /*! \brief Destructor, frees any freeable sprites and destroys list */
  void SpriteList::Destroy();
  /*! \brief Attempts to removes specified sprite from list           */
  bool Extract(Sprite *spr);
  /*! \brief Attempts to insert specified sprite into the list        */
  bool Insert(Sprite *spr);
  /*! \brief Attempts to find the specified sprite in the list        */
  bool Find(Sprite *spr);
  /*! \brief Returns the last sprite in the list, or NULL if none     */
  Sprite *Tail();
  /*! \brief Returns the first sprite in the list, or NULL if none    */
  Sprite *Head();  
private:
  /*! \brief Attempts to append a sprite to the end of the list       */
  bool Append(Sprite *spr);
  /*! \brief Attempts to insert a sprite in at the front of the list  */
  bool Prepend(Sprite *spr);
  /*! \brief Pointer to the first sprite in the list  */
  Sprite *head;
};  

#include <SDL/close_code.h>

#endif
