/***************************************************************************
                              toyparse.h
    Functions used in the Toy Memory Manager String Parser.

    How the parser works:
      At the root of it all, is the command table.  It contains
    a list of pointers;  each pointer points to a list of strings.
    Both the table and each list in it are NULL-terminated.    
      The strings denote valid commands or valid command forms.
    For instance, say you have the root category of 'value'.  Being
    the first thing in the command table, it would be at index 0,
    and point to a list of strings, like { "#%d","\"%s",NULL };
      %d means accept any sequence of numbers in that spot, %s means
    accept any integer values in that spot.  So both "#100" and "\"Hello"
    would be accepted as valid commands.  "#100" would be subcommand
    0, while "\"Hello" would be subcommand 1.
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

#ifndef __TOYPARSE_H__
#define __TOYPARSE_H__

#ifdef __cplusplus
  extern "C"  {
#endif/*__cplusplus*/

#include <SDL/begin_code.h>

/**
 * \defgroup Parser Generic String Parser
 *   How the parser works:
 *
 *     At the root of it all, is the command table.  It contains
 *   a list of pointers;  each pointer points to a list of strings.
 *   Both the table and each list in it are NULL-terminated.    
 *     The strings denote valid commands or valid command forms.
 *   For instance, say you have the root category of 'value'.  Being
 *   the first thing in the command table, it would be at index 0,
 *   and point to a list of strings, like { "#%d","\"%s",NULL };
 *     %d means accept any sequence of numbers in that spot, %s means
 *   accept any integer values in that spot.  So both "#100" and "\"Hello"
 *   would be accepted as valid commands.  "#100" would be subcommand
 *   0, while "\"Hello" would be subcommand 1.
 */
/*@{*/

/*! \brief A structure defining the type and subtype of a token. */
/**
 *  When used in arrays, if token==NULL that is the last element.
 */
typedef struct TokenType
{
  /*! \brief Type of token */
  int type;
  /*! \brief Subtype of token */
  int subtype;  
  /*! \brief The token itself */
  char *token;
} TokenType;

/*! \brief Frees an array of token-types and everything it contains */
DECLSPEC void SDLCALL FreeType(TokenType *types);
/*! \brief Converts a line of text to an array of token-types */
DECLSPEC TokenType *SDLCALL LineToTypes(const char *str, const char **table[]);

/*! \brief Determines the root type of a token */
DECLSPEC int SDLCALL DetermineType(const char *str, const char **table[]);

/*! \brief Determines the sub-type from a known type */
DECLSPEC int SDLCALL DetermineSubType(int type,const char *str, const char **args[]);                                    

DECLSPEC int SDLCALL CheckLineSyntax(TokenType *tarray, ...);

/*! \brief Denotes an open-ended syntax check */
#define TYPE_ENDOPEN  0xffff
/*! \brief Denotes a strict syntax-check, last argument MUST be the last */
#define TYPE_ENDCLOSE 0xfffe

/*@}*/

#include <SDL/close_code.h>

#ifdef __cplusplus
  }
#endif/*__cplusplus*/

#endif/*__TOYPARSE_H__*/
