/***************************************************************************
                          spritelibtest.cpp  -  description
                             -------------------
    begin                : Sun Jun 8 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <spritelib.h>
#include "screen.h"
#include "bouncy.h"

#ifdef _WIN32_WCE
#  define VFLAGS (SDL_ANYFORMAT|SDL_FULLSCREEN)
#  define SPRITE_FILE_DIR "\\"
/* Yeah, Windows CE framerates suck.  But at least it's not my fault. */
#  define FPS 20
#else
#  define VFLAGS (SDL_ANYFORMAT)
#  define SPRITE_FILE_DIR "/usr/share/spritelib/"
#  define FPS 100
#endif
#define NUM_FACES       5

void TestFxp();

/**
 *  A brief and simple test of SpriteLib
 */
int main(int argc, char *argv[])
{
  int n=-1;  
  /* SpriteManager object */
  SpriteManager sm(SCREEN_W,SCREEN_H,SCREEN_BPP,VFLAGS);
  /* Background image     */
  SpriteLoader  bg("background.spr",SPRITE_FILE_DIR);
  /* Bouncing happyface   */
  SpriteLoader  faces("faces.spr",SPRITE_FILE_DIR);
  
  if(bg.Good())
  { /*  Blit background image onto SM's internal background */
    sm.DrawOnBG(bg.GetSurface(),NULL,NULL);
  }
  else
  {
    fprintf(stderr,"Couldn't load %sbackground.spr\n",SPRITE_FILE_DIR);
    return(1);
  }

  TestFxp();

  if(faces.Good())
  {
    /* Add bouncy sprites to to the game  */  
    for(n=0; n<NUM_FACES; n++)
    {
      Sprite *spr=faces.GetSprite(new Bouncy());            
      sm.AddSprite(spr);
    }
  }
  else
  {
    fprintf(stderr,"Couldn't load %sfaces.spr\n",SPRITE_FILE_DIR);
    return(1);
  }
  
  sm.Run(1000/FPS); /* Run event loop until SDL_QUIT  */
  
  return(0);
}

void TestFxp()
{
  int n;

  Sfxp a=TO_SFXP(2);
  Sfxp b=TO_SFXP(3);

  printf("Testing some basic math, in fixed point:\n");

  printf("%d + %d = %d\n",FROM_SFXP(a),FROM_SFXP(b),FROM_SFXP(a+b));
  printf("%d - %d = %d\n",FROM_SFXP(a),FROM_SFXP(b),FROM_SFXP(a-b));
  printf("%d * %d = %d\n",FROM_SFXP(a),FROM_SFXP(b),FROM_SFXP(MUL_SFXP(a,b)));
  printf("%d / %d = %f\n",FROM_SFXP(a),FROM_SFXP(b),SFXP_TO_F(DIV_SFXP(a,b)));
  printf("%.3f ~=%d\n",1.499,FROM_SFXP(F_TO_SFXP(1.499)));
  printf("%.3f ~=%d\n",1.500,FROM_SFXP(F_TO_SFXP(1.500)));

  printf("\nTesting trig:\n");
  for(n=0; n<360; n+=45)
  {
    printf("Cos(%3d deg.) = %f\n",n%360,SFXP_TO_F(FXP_Cos(DEG_TO_AFXP(n))));
    printf("Sin(%3d deg.) = %f\n",n%360,SFXP_TO_F(FXP_Sin(DEG_TO_AFXP(n))));
    printf("Tan(%3d deg.) = %f\n",n%360,SFXP_TO_F(FXP_Tan(DEG_TO_AFXP(n))));
  }

  printf("\nTesting trajectory angle:\n");
  for(n=0; n<360; n+=(360/12))
  {
    Afxp a=DEG_TO_AFXP(n);
    Afxp na=FXP_Traj(FXP_Sin(a),FXP_Cos(a));

    printf("%3d degrees -> %3d degrees\n",n%360,(int)AFXP_TO_DEG(na));
  }
}
